#!/usr/bin/perl


package SMART;

use strict;
use lib qw(/usr/sausalito/perl);
use vars qw(@ISA @EXPORT);

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_smart_info smart_to_array array_to_smart);


# Runs ide-smart with and returns 2 has references.
# The first hash reference contains the Id and Value data and has the following structure:
# hash->{drive}->{id} = value
#             |->{id} = value
#
# The second hash reference contains the failure state of the Ids and has the following
# structure:
#
# hash->{drive}->{id} = boolean FAILED
#             |->{id} = boolean
# The boolean equals 1 if the id has failed

1;

sub get_smart_info {
    my @drives = @_;
    my $data = undef;
    my $failures = undef;
    my ($id, $val);
    
    foreach my $drive (@drives) {
	open(SMART, "/usr/local/sbin/ide-smart $drive 2>&1 |") or die "can't run ide-smart from get_smart_info";
	while (<SMART>) {
	    if (/Id=\s*(\d+).*Value=\s*(\d+)/) {
		$id = $1;
		$val = $2;
		$data->{$drive}->{$id} = $val;
		if (/Failed/) {
		    $failures->{$drive}->{$id} = 1;
		}
	    }
	}
	close(SMART);
    }

    if (wantarray) {
	return ($failures, $data);
    } else {
	return $data;
    }
}


# takes hash reference of smart info and returns an array of elements 
# of form drive/id/value
sub smart_to_array {
    my $hash = shift;
    my @drives = keys(%$hash);
    my @result = ();
    foreach my $drive (@drives) {
	foreach my $id (keys %{$hash->{$drive}}) {
	    push @result, "$drive/$id/" . $hash->{$drive}->{$id};
	}
    }
    return @result;

}

# takes array of elements of form drive/id/value and returns hash ref
# hash->{drive}->{id}->{value}
sub array_to_smart {
    my $hash = undef;
    my @fields;
    foreach my $entry (@_) {
	@fields = split('/', $entry);
	$hash->{$fields[0]}->{$fields[1]} = $fields[2];
    }
    return $hash;
}
